var powerOn = 0;
var selectedTone = 0;
var pitch = 0;
var updateTimer = null;
var volume = 0;
var targetFrequency = 0;
var transpose = 1.0;
var inOnOffAnimation = 0;
var selectedTuning = "Standard";
var noteFrequencies = new Array(82.407, 110.000, 146.832, 195.998, 246.942, 329.628);
var noteNames = new Array ("E", "A", "d", "g", "b", "e'");

function loaded() {
	var drawer = document.getElementById("drawer-image");
	document.addEventListener("keydown", keyPressed, true);
	new AppleInfoButton(document.getElementById("info-button"), document.getElementById("drawer"), "white", "white", showPrefs);
	new AppleGlassButton(document.getElementById("done-button"), "Done", hidePrefs);
	if (!TunerPlugin) return;
	TunerPlugin.setOutputAmplitude_(volume/3.0);
	TunerPlugin.setTranspose_(1.0);
	
	if (window.widget) {
		widget.onshow = onShow;
		widget.onhide = onHide;
		
		prefValue = widget.preferenceForKey ("tuning");
		if (prefValue && prefValue.length > 0) setTuning(prefValue);
		
		prefValue = widget.preferenceForKey ("volume");
		if (prefValue && prefValue.length > 0) setVolume(parseFloat(prefValue));
		
		prefValue = widget.preferenceForKey ("transpose");
		if (prefValue && prefValue.length > 0) setTranspose(parseFloat(prefValue));
	}
}

function onShow () {
}

function onHide () {
	turnOffPower(0);
}

var drawerAnimationHandler = function (animation, current, start, finish) {
	drawer.style.left = (400 * current) + "px";
}

function togglePower() {
	if (inOnOffAnimation) return;
	if (powerOn == 0) 
		turnOnPower(1);
	else
		turnOffPower(1);
}

function turnOnPower(animate) {
	if (powerOn == 1) return;
	powerOn = 1;
	document.getElementById ("power-button-image").src = "Images/power-on.png";
	
	if (TunerPlugin) TunerPlugin.turnOn();
	if (updateTimer == null) updateTimer = setInterval("fireUpdateTimer();", 66);
	window.resizeTo (480, 167);
	
	if (animate) {
		new AppleAnimator(500, 13, -1.0, 0.0, drawerAnimationHandler).start();
		inOnOffAnimation = 1;
		setTimeout ("inOnOffAnimation = 0;", 500);
	}
	pitch = 0.0;
	fireUpdateTimer();
}

function turnOffPower(animate) {
	if (powerOn == 0) return;
	powerOn = 0;
	selectToneButton(0);
	if (updateTimer != null) {
		clearInterval(updateTimer);
		updateTimer = null;
	}
	document.getElementById ("power-button-image").src = "Images/power-off.png";
	if (animate) {
		new AppleAnimator(500, 13, 0.0, -1.0, drawerAnimationHandler).start();
		inOnOffAnimation = 1;
		setTimeout ("window.resizeTo (68, 167); inOnOffAnimation = 0;", 500);
	} else {
		drawer.style.left = "-400px";
		window.resizeTo (68, 167);
	}
	setTimeout ("TunerPlugin.turnOff();", 100);
}

function openSoundStudio() {
	TunerPlugin.openSoundStudio();
}

function toggleTranspose() {
	setTranspose (transpose <1.5? 2.0: 1.0);
}

function setTranspose (x) {
	if (transpose != x) {
		transpose = x;
		if (transpose > 1.5)
			document.getElementById ("transpose-image").src = "Images/8va-h.png";
		else
			document.getElementById ("transpose-image").src = "Images/8va.png";
		if (TunerPlugin) TunerPlugin.setTranspose_(transpose);
		if (window.widget) widget.setPreferenceForKey(transpose.toString(), "transpose");
	}
}

function setVolume (x) {
	if (volume != x) {
		volume = x;
		document.getElementById ("speaker-image").src = "Images/speaker-"+volume+".png";
		y = volume/3.0;
		TunerPlugin.setOutputAmplitude_(y*y);
		if (window.widget) widget.setPreferenceForKey(volume.toString(), "volume");
	}
}

function increaseVolume() {
	setVolume (volume>=3? 0: volume+1);
}

function selectToneButton (x) {
	selectedTone = x;
	for (i=0; i<=6; i++) {
		item = document.getElementById ("tone-"+i);
		
		if (i==0) baseFilename = "url(Images/segment-l";
		else if (i==6) baseFilename = "url(Images/segment-r";
		else baseFilename = "url(Images/segment-26px";
		
		if (item) {
			if (i == x) {
				item.style.backgroundImage = baseFilename+"-h.png)";
				item.style.color = "#000";
			} else {
				item.style.backgroundImage = baseFilename+".png)";
				item.style.color = "#FFF";
			}
		}
	}
	
	if (window.TunerPlugin) {
		if (x == 0) {
			targetFrequency = 0; 
			targetNote = "";
		} else if (x <= 6) {
			targetFrequency = noteFrequencies[x-1];
			targetNote = noteNames[x-1];
		}

		TunerPlugin.setTargetFrequency_(targetFrequency);
		document.getElementById ("note").innerText = targetNote;
		
		if (targetFrequency == 0) {
			document.getElementById ("note-mode").innerText = "automatic";
		} else {
			document.getElementById ("note-mode").innerText = "target";
		}
	}
}

function keyPressed (event) {
	if (event.metaKey) return; // meta key == command key
	
	if (event.charCode == 27) // esc
		togglePower();
	else if (powerOn == 0)
		return;
	
	if (event.keyIdentifier == "Right") {
		selectToneButton(selectedTone>=6? 0: selectedTone+1);
	} else if (event.keyIdentifier == "Left") {
		selectToneButton(selectedTone<=0? 6: selectedTone-1);
	} else if ((48 <= event.charCode) && (event.charCode <= 54)) {
		selectToneButton(event.charCode-48);
	} else {
		switch (event.charCode) {
			case 32: selectToneButton(selectedTone >=6? 0: selectedTone+1); break; // space
			case 0x60: selectToneButton(0); break; // `
			case 0x4D: case 0x6D: setVolume(0); break; // M
			case 0x2B: case 0x3D: if (volume < 3) setVolume (volume+1); break; // +=
			case 0x2D: if (volume > 0) setVolume (volume-1); break; // -
			case 0x54: case 0x74: toggleTranspose(); break; // T
			default: break;
		}
	}
	event.stopPropagation();
	event.preventDefault();
}

function fireUpdateTimer () {
	if (powerOn == 0) return;
	
	// -- Confidence --
	confidenceValue = TunerPlugin.inputConfidence();
	canvas = document.getElementById ("confidence-canvas");
	if (canvas) {
		context = canvas.getContext("2d");
		context.clearRect(0,0,8,24);
		context.fillStyle = "#FFFFFF";
		h = 22*confidenceValue;
		context.fillRect(0,22-h,8,h);
	}

	// -- Pitch --
	if (targetFrequency == 0) {
		document.getElementById ("note").innerText = TunerPlugin.inputNoteName();
	}
	
	currentPitch = TunerPlugin.inputNotePitch() / 50;
	if ((-24 <= currentPitch) && (currentPitch <= 24)) {
		y = confidenceValue * 0.8;
		pitch = pitch * (1.0 - y) + currentPitch * y;
		if (pitch < -1.0) pitch = -1.0;
		if (pitch >  1.0) pitch =  1.0;
		x = 210 + pitch * 155 - 24;
		document.getElementById ("indicator").style.left = x.toString()+"px";
	}

	// -- Levels --
	levelsImage = document.getElementById ("levels-image");
	if (levelsImage) {
		peakLevel = TunerPlugin.inputPeakLevel();
		if (peakLevel < -32.95) 
			levelsImage.src = "Images/levels-0.png";
		else if (peakLevel < -27) 
			levelsImage.src = "Images/levels-1.png";
		else if (peakLevel < -20) 
			levelsImage.src = "Images/levels-2.png";
		else if (peakLevel < -10) 
			levelsImage.src = "Images/levels-3.png";
		else if (peakLevel < -2.5) 
			levelsImage.src = "Images/levels-4.png";
		else 
			levelsImage.src = "Images/levels-5.png";
	}
}

function showPrefs () {
	if (!window.widget) return;
	
	selectToneButton(0);
	if (selectedTuning == "Drop D") {
		document.getElementById("tuning-popup").selectedIndex = 1;
	} else if (selectedTuning == "Drop C") {
		document.getElementById("tuning-popup").selectedIndex = 2;
	} else if (selectedTuning == "Open G") {
		document.getElementById("tuning-popup").selectedIndex = 3;
	} else if (selectedTuning == "Open D") {
		document.getElementById("tuning-popup").selectedIndex = 4;
	} else {
		document.getElementById("tuning-popup").selectedIndex = 0;
	}
	
	widget.prepareForTransition("ToBack");
	document.getElementById("front").style.display="none";
	document.getElementById("back").style.display="block";
	setTimeout ('widget.performTransition();', 0);
}

function hidePrefs () {
	newTuning = "Standard";
	switch (document.getElementById("tuning-popup").selectedIndex) {
		case 1: newTuning = "Drop D"; break;
		case 2: newTuning = "Drop C"; break;
		case 3: newTuning = "Open G"; break;
		case 4: newTuning = "Open D"; break;
		default: break;
	}
	setTuning (newTuning);
	
	if (!window.widget) return;
	widget.prepareForTransition("ToBack");
	document.getElementById("back").style.display="none";
	document.getElementById("front").style.display="block";
	setTimeout ('widget.performTransition();', 0);
}

function setTuning (newTuning) {
	if (selectedTuning != newTuning) {
		selectedTuning = newTuning;
		if (selectedTuning == "Standard") {
			noteFrequencies = new Array(82.407, 110.000, 146.832, 195.998, 246.942, 329.628);
			noteNames = new Array ("E", "A", "d", "g", "b", "e'");
		} else if (selectedTuning == "Drop D") {
			noteFrequencies = new Array(73.416, 110.000, 146.832, 195.998, 246.942, 329.628);
			noteNames = new Array ("D", "A", "d", "g", "b", "e'");
		} else if (selectedTuning == "Drop C") {
			noteFrequencies = new Array(65.406, 97.999, 130.813, 174.614, 220.000, 293.665);
			noteNames = new Array ("C", "G", "c", "f", "a", "d'");
		} else if (selectedTuning == "Open G") {
			noteFrequencies = new Array(73.416, 97.999, 146.832, 195.998, 246.942, 293.665);
			noteNames = new Array ("D", "G", "d", "g", "b", "d'");
		} else if (selectedTuning == "Open D") {
			noteFrequencies = new Array(73.416, 110.000, 146.832, 184.997, 220.000, 293.665);
			noteNames = new Array ("D", "A", "d", "f♯", "a", "d'");
		}
		for (i=1; i<=6; i++) {
			item = document.getElementById ("tone-"+i);
			item.innerText = noteNames[i-1];
		}
		selectToneButton(0);
		if (window.widget) widget.setPreferenceForKey(selectedTuning, "tuning");
		if (selectedTuning == "Standard") {
			document.getElementById("title-text").innerHTML = "Tuna Pitch";
		} else {
			document.getElementById("title-text").innerHTML = selectedTuning;
		}
	}
}